@@ -113,8 +113,8 @@ impl IceHeader {
 #[repr(C)]
 pub(crate) struct IceGroupHeaders {
     pub groups: [IceGroupHeader; 2],
-    pub group1_size: U32<LE>,
-    pub group2_size: U32<LE>,
+    pub group1_shuffled_size: U32<LE>,
+    pub group2_shuffled_size: U32<LE>,
     pub key: U32<LE>,
     pub _reserved: U32<LE>,
 }
@@ -271,13 +271,13 @@ impl IceArchive {
 
             if ice_info.flags.get() & 0x1 != 0 {
                 // Decrypt group data
-                let mut key = group_header.group1_size.get();
+                let mut key = group_header.group1_shuffled_size.get();
                 if key != 0 {
                     key.swap_bytes();
                 } else {
                     key = group_header.groups[0].size.get()
                         ^ group_header.groups[1].size.get()
-                        ^ group_header.group2_size.get()
+                        ^ group_header.group2_shuffled_size.get()
                         ^ group_header.key.get()
                         ^ 0xC8D7469A;
                 }
@@ -299,7 +299,7 @@ impl IceArchive {
             src.read_exact(&mut buf[..])?;
             ice_info = *LayoutVerified::<_, IceInfo>::new_unaligned(&buf[..]).unwrap();
             if version > 4 {
-                src.seek(SeekFrom::Current(10))?;
+                src.seek(SeekFrom::Current(0x10))?;
             }
             let mut table = [0u8; 0x100];
             src.read_exact(&mut table[..])?;
