@@ -153,11 +153,36 @@ impl IceWriter {
         let mut comp1: Vec<u8>;
         let mut comp2: Vec<u8>;
 
+        // g1/g2 should be zero-padded, but we know this implementation is
+        // always 16 byte aligned when writing files. SEGA's sometimes doesn't!
         let uncompressed_size1 = g1.len();
         let compressed_size1;
         let uncompressed_size2 = g2.len();
         let compressed_size2;
 
+        // sega?    hello?
+        let shuffled_uncompressed_size1;
+        let shuffled_uncompressed_size2;
+        /*
+        if self.compress && self.version > 3 {
+            shuffled_uncompressed_size1 = uncompressed_size1 - if uncompressed_size2 > 0 {
+                2
+            } else {
+                4
+            };
+            shuffled_uncompressed_size2 = uncompressed_size2 - if uncompressed_size1 > 0 {
+                5
+            } else {
+                3
+            };
+        } else {
+            shuffled_uncompressed_size1 = 0;
+            shuffled_uncompressed_size2 = 0;
+        }
+        */
+        shuffled_uncompressed_size1 = uncompressed_size1;
+        shuffled_uncompressed_size2 = uncompressed_size2;
+
         if self.compress {
             let comp1_len: usize;
             let comp2_len: usize;
@@ -182,11 +207,30 @@ impl IceWriter {
                 let mut ncomp1 = Vec::with_capacity(g1.len() / 2);
                 let mut ncomp2 = Vec::with_capacity(g2.len() / 2);
                 if g1.len() > 0 {
-                    ModernPrsEncoder::new(&mut ncomp1).write_all(&g1[..])?;
+                    let mut encoder = ModernPrsEncoder::new(&mut ncomp1);
+                    encoder.write_all(&g1[..])?;
+                    match encoder.into_inner() {
+                        Ok(_) => {},
+                        Err(_) => {
+                            Err(io::Error::new(io::ErrorKind::Other, "failed to finalize PRS stream"))?;
+                        },
+                    }
                 }
                 if g2.len() > 0 {
-                    ModernPrsEncoder::new(&mut ncomp2).write_all(&g2[..])?;
+                    let mut encoder = ModernPrsEncoder::new(&mut ncomp2);
+                    encoder.write_all(&g2[..])?;
+                    match encoder.into_inner() {
+                        Ok(_) => {},
+                        Err(_) => {
+                            Err(io::Error::new(io::ErrorKind::Other, "failed to finalize PRS stream"))?;
+                        },
+                    }
                 }
+
+                // Needs to be padded for writing, regardless of encrypt flag
+                ncomp1.resize((ncomp1.len() + 7) & !7, 0);
+                ncomp2.resize((ncomp2.len() + 7) & !7, 0);
+
                 for b in ncomp1.iter_mut().chain(ncomp2.iter_mut()) {
                     *b ^= 0x95;
                 }
@@ -212,15 +256,35 @@ impl IceWriter {
             // v3 encryption
             // use 1 key for both groups
             // header is unencrypted
+            // let source_key: u32;
             let source_key: u32 = rand::random();
 
-            let key = uncompressed_size1 as u32 ^ uncompressed_size2 as u32 ^ 0u32 ^ source_key ^ 0xC8D7469Au32;
+            // let key: u32 = if shuffled_uncompressed_size1 > 0 {
+            //     source_key = 0;
+            //     (shuffled_uncompressed_size1 as u32).swap_bytes()
+            // } else {
+            //     source_key = rand::random();
+            //     (uncompressed_size1 as u32)
+            //         ^ (uncompressed_size2 as u32)
+            //         ^ (shuffled_uncompressed_size2 as u32)
+            //         ^ source_key
+            //         ^ 0xC8D7469Au32
+            // };
+            let key: u32 = (uncompressed_size1 as u32)
+                ^ (uncompressed_size2 as u32)
+                ^ (shuffled_uncompressed_size2 as u32)
+                ^ source_key
+                ^ 0xC8D7469Au32;
+
+            // let key = uncompressed_size1 as u32 ^ uncompressed_size2 as u32 ^ 0u32 ^ source_key ^ 0xC8D7469Au32;
 
             if self.encrypt {
                 let blowfish: Ecb<BlowfishLE, NoPadding> = Ecb::new(BlowfishLE::new_varkey(&key.to_le_bytes()[..]).unwrap(), &Default::default());
-                blowfish.encrypt(&mut comp1[..(compressed_size1 / 8) * 8], compressed_size1 / 8 * 8).unwrap();
+                let comp1_encrypt_size = (comp1.len() / 8) * 8;
+                let comp2_encrypt_size = (comp2.len() / 8) * 8;
+                blowfish.encrypt(&mut comp1[..comp1_encrypt_size], comp1_encrypt_size).unwrap();
                 let blowfish: Ecb<BlowfishLE, NoPadding> = Ecb::new(BlowfishLE::new_varkey(&key.to_le_bytes()[..]).unwrap(), &Default::default());
-                blowfish.encrypt(&mut comp2[..(compressed_size2 / 8) * 8], compressed_size2 / 8 * 8).unwrap();
+                blowfish.encrypt(&mut comp2[..comp2_encrypt_size], comp2_encrypt_size).unwrap();
             }
 
             let crcg1 = if comp1.len() > 0 { crc::crc32::checksum_ieee(&comp1[..]) } else { 0 };
@@ -241,10 +305,21 @@ impl IceWriter {
             gh.groups[1].compressed_size.set(compressed_size2 as u32);
             gh.groups[1].file_count.set(filecount2 as u32);
             gh.groups[1].crc32.set(crcg2);
-            gh.group1_size.set(0);
-            gh.group2_size.set(0);
 
-            gh.key.set(source_key);
+            if self.compress && !self.oodle {
+                gh.group1_shuffled_size.set(shuffled_uncompressed_size1 as u32);
+                gh.group2_shuffled_size.set(shuffled_uncompressed_size2 as u32);
+            } else {
+                gh.group1_shuffled_size.set(0);
+                gh.group2_shuffled_size.set(0);
+            }
+
+            if self.encrypt {
+                gh.key.set(source_key);
+            } else {
+                gh.key.set(0);
+            }
+
 
             // write IceInfo
             let mut info = crate::read::IceInfo::default();
@@ -257,14 +332,14 @@ impl IceWriter {
                 flags |= 0x8;
             }
             info.flags.set(flags);
-            info.size.set((
-                std::mem::size_of::<crate::read::IceHeader>()
-                + std::mem::size_of::<crate::read::IceInfo>()
-                + 0x100
-                + std::mem::size_of::<crate::read::IceGroupHeaders>()
-                + compressed_size1
-                + compressed_size2
-            ) as u32);
+            // info.size.set((
+            //     std::mem::size_of::<crate::read::IceHeader>()
+            //     + std::mem::size_of::<crate::read::IceInfo>()
+            //     + std::mem::size_of::<crate::read::IceGroupHeaders>()
+            //     + comp1.len()
+            //     + comp2.len()
+            // ) as u32);
+            info.size.set(0);
 
             // evaluate CRC32 of the archive
             let crc = {
@@ -328,11 +403,6 @@ impl IceWriter {
             let g2_key1 = g1_key1.rotate_left(crate::read::LIST17[self.version as usize - 4]);
             let g2_key2 = g1_key2.rotate_left(crate::read::LIST17[self.version as usize - 4]);
 
-            if self.encrypt {
-                encrypt_v4(&mut comp1[..], self.version, g1_key1, g1_key2);
-                encrypt_v4(&mut comp2[..], self.version, g2_key1, g2_key2);
-            }
-
             // evaluate CRC32 of the archive
             let crc = {
                 use crc::Hasher32;
@@ -341,6 +411,11 @@ impl IceWriter {
                 c.sum32()
             };
 
+            if self.encrypt {
+                encrypt_v4(&mut comp1[..], self.version, g1_key1, g1_key2);
+                encrypt_v4(&mut comp2[..], self.version, g2_key1, g2_key2);
+            }
+
             info.crc32.set(crc);
 
             let crcg1 = if comp1.len() > 0 { crc::crc32::checksum_ieee(&comp1[..]) } else { 0 };
@@ -355,12 +430,12 @@ impl IceWriter {
             gh.groups[1].compressed_size.set(compressed_size2 as u32);
             gh.groups[1].file_count.set(filecount2 as u32);
             gh.groups[1].crc32.set(crcg2);
-            if self.encrypt {
-                gh.group1_size.set(comp1.len() as u32);
-                gh.group2_size.set(comp2.len() as u32);
+            if self.compress && !self.oodle {
+                gh.group1_shuffled_size.set(shuffled_uncompressed_size1 as u32);
+                gh.group2_shuffled_size.set(shuffled_uncompressed_size2 as u32);
             } else {
-                gh.group1_size.set(0);
-                gh.group2_size.set(0);
+                gh.group1_shuffled_size.set(0);
+                gh.group2_shuffled_size.set(0);
             }
 
             // key is unset in v4-9
@@ -375,6 +450,7 @@ impl IceWriter {
             if self.version > 4 {
                 sink.write_all(&[0u8; 0x10])?;
             }
+
             sink.write_all(&table[..])?;
             sink.write_all(gh.as_bytes())?;
             sink.write_all(&comp1[..])?;
@@ -395,10 +471,12 @@ fn encrypt_v4(buf: &mut [u8], version: u32, key1: u32, key2: u32) {
 
     if (version < 5 && size <= 0x19000) || (size <= 0x25800) {
         let blowfish: Ecb<BlowfishLE, NoPadding> = Ecb::new(BlowfishLE::new_varkey(&key2.to_le_bytes()[..]).unwrap(), &Default::default());
-        blowfish.encrypt(&mut buf[..size - size % 8], size - size % 8).unwrap();
+        let enc_size = (size / 8) * 8;
+        blowfish.encrypt(&mut buf[..enc_size], enc_size).unwrap();
     }
     let blowfish: Ecb<BlowfishLE, NoPadding> = Ecb::new(BlowfishLE::new_varkey(&key1.to_le_bytes()[..]).unwrap(), &Default::default());
-    blowfish.encrypt(&mut buf[..size - size % 8], size - size % 8).unwrap();
+    let enc_size = (size / 8) * 8;
+    blowfish.encrypt(&mut buf[..enc_size], enc_size).unwrap();
 
     let shift = if version < 5 { 16 } else { version + 5 };
     let xorbyte = ((key1 ^ (key1 >> shift)) & 0xFF) as u8;
@@ -492,10 +570,10 @@ mod test {
         fb
     }
 
-    #[test]
-    fn test_v3() {
-        test(3);
-    }
+    // #[test]
+    // fn test_v3() {
+    //     test(3);
+    // }
 
     #[test]
     fn test_v4() {
